@@ -1,3 +1,4 @@
+# $Id: POE.pm 2314 2008-04-19 22:18:35Z rcaputo $
 # Copyrights and documentation are after __END__.
 
 package POE;
@@ -5,20 +6,21 @@ package POE;
 use strict;
 use Carp qw( croak );
 
-use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+use vars qw($VERSION $REVISION);
+$VERSION = '1.0001'; # NOTE - Should be #.#### (four decimal places)
+$REVISION = do {my($r)=(q$Revision: 2314 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 sub import {
   my $self = shift;
 
-  my @loops    = grep(/^(?:XS::)?Loop::/, @_);
+  my @loops    = grep(/^Loop\:\:/, @_);
   my @sessions = grep(/^(Session|NFA)$/, @_);
-  my @modules  = grep(!/^(Kernel|Session|NFA|(?:XS::)?Loop::[\w:]+)$/, @_);
+  my @modules  = grep(!/^(Kernel|Session|NFA|Loop)$/, @_);
 
   croak "can't use multiple event loops at once"
     if (@loops > 1);
   croak "POE::Session and POE::NFA export conflicting constants"
-    if scalar @sessions > 1;
+    if grep(/^(Session|NFA)$/, @sessions) > 1;
 
   # If a session was specified, use that.  Otherwise use Session.
   if (@sessions) {
@@ -70,7 +72,7 @@ __END__
 
 =head1 NAME
 
-POE - portable multitasking and networking framework for any event loop
+POE - portable multitasking and networking framework for Perl
 
 =head1 SYNOPSIS
 
@@ -115,31 +117,27 @@ POE is a framework for cooperative, event driven multitasking and
 networking in Perl.  Other languages have similar frameworks.  Python
 has Twisted.  TCL has "the event loop".
 
+POE originally was developed as the core of a persistent object server
+and runtime environment.  It has since evolved into something much
+more generic and widely useful.
+
 POE provides a unified interface for several other event loops,
-including select(), L<IO::Poll|IO::Poll>, L<Glib>, L<Gtk>, L<Tk>,
-L<Wx>, and L<Gtk2>.  Many of these event loop interfaces were written
-by others, with the help of POE::Test::Loops.  They may be found on
-the CPAN.
+including select(), L<IO::Poll|IO::Poll>, L<Glib>, L<Gtk>, L<Tk>, L<Wx>, 
+L<Gtk2>, and so on.
+Check the CPAN for the full list of L<POE::Loop|POE::Loop> modules.
 
-POE achieves its its high degree of portability to different operating
-systems and Perl versions by being written entirely in Perl.  CPAN
-hosts optional XS modules for POE if speed is more desirable than
-portability.
-
-POE is designed in layers.  Each layer builds atop the lower level
-ones.  Programs are free to use POE at any level of abstraction, and
+POE is designed in layers, each building atop the lower level ones.
+Programs are free to use POE at any level of abstraction, and
 different levels can be mixed and matched seamlessly within a single
-program.  Remember, though, that higher-level abstractions often
-require more resources than lower-level ones.  The conveniences they
-provide are not free.
+program.
 
 POE's bundled abstraction layers are the tip of a growing iceberg.
-L<Sprocket>, L<POE::Stage|POE::Stage>, and other CPAN distributions
-build upon this work.  You're encouraged to look around.
+L<Sprocket>, L<POE::Stage|POE::Stage>, and other CPAN distributions build upon this
+work.  You're encouraged to look around.
 
 No matter how high you go, though, it all boils down to calls to
-L<POE::Kernel|POE::Kernel>.  So your down-to-earth code can easily
-cooperate with stratospheric systems.
+L<POE::Kernel|POE::Kernel>.  So your down-to-earth code can easily cooperate
+with stratospheric systems.
 
 =head2 Layer 1: Kernel and Sessions
 
@@ -270,9 +268,9 @@ classes describe what's common among each class.  It's a good idea to at
 least skim the base class documentation since the subclasses tend not to
 rehash the common things.
 
-L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, and L<POE::Loop|POE::Loop> document the
+L<POE::Queue|POE::Queue>, L<POE::Resource|POE::Resource>, L<POE::Loop|POE::Loop>, and L<POE::API|POE::API> document the
 concepts and sometimes the standard interfaces behind multiple
-subclasses.  You're encouraged to have a look.
+subclasess.  You're encouraged to have a look.
 
 =head2 Helper Classes
 
@@ -355,7 +353,7 @@ non-obvious detail.
 
 One of POE's design goals is to be as portable as possible.  That's
 why it's written in "Plain Perl".  XS versions of POE modules are
-available as third-party distributions.  Parts of POE that require
+avaliable as third-party distributions.  Parts of POE that require
 nonstandard libraries are optional, and not having those libraries
 should not prevent POE from installing.
 
@@ -365,9 +363,10 @@ see the GETTING HELP section if you run into a problem.
 POE is expected to work on most forms of UNIX, including FreeBSD,
 MacOS X, Linux, Solaris.  Maybe even AIX and QNX, but we're not sure.
 
-POE is also tested on Windows XP, using the latest version of
-ActiveState, Strawberry and Cygwin Perl.  POE is fully supported with
-Strawberry Perl, as it's included in the Strawberry distribution.
+POE has also been tested on Windows XP, using the latest version of
+ActiveState Perl and Cygwin Perl.  Support for the Neopolitan Perls
+(Vanilla, Strawberry and Chocolate) is still early, but there should
+be no major showstoppers.
 
 OS/2 and MacOS 9 have been reported to work in the past, but nobody
 seems to be testing there anymore.  Reports and patches are still
@@ -383,7 +382,7 @@ POE's quality is due in large part to the fine work of Chris Williams
 and the other CPAN testers.  They have dedicated resources towards
 ensuring CPAN distributions pass their own tests, and we watch their
 reports religiously.  You can, too.  The latest POE test reports can
-be found at L<http://cpantesters.org/distro/P/POE.html>.
+be found at <http://testers.cpan.org/search?request=dist&dist=POE>.
 
 Thanks also go out to Benjamin Smith and the 2006 Google Summer of
 Code.  Ben was awarded a grant to improve POE's test suite, which he
@@ -405,26 +404,6 @@ through contributions of time, patches, and other resources.  Some of
 them are: Sean Puckett, Douglas Couch, Andrew Chen, Uhlarik Ondoej,
 Nick Williams, and Chris Williams (no relation).
 
-=head2 Linux/Unix Issues
-
-=head3 pty woes
-
-Some distributions chose to not completely setup the pseudo-tty
-support. This is needed for L<POE::Wheel::Run> to interact with the
-subprocess. If you see something like this while running C<make test>
-please look at your distribution's documentation on how to fix it. For
-example, on Debian-based systems the solution was to execute 
-"sudo apt-get install udev".
-
-  t/30_loops/io_poll/wheel_run.t ..................... 1/99
-  pty_allocate(nonfatal): posix_openpt(): No such file or directory at /usr/local/lib/perl/5.10.0/IO/Pty.pm line 24.
-  ...
-  Cannot open a pty at /home/apoc/poe/blib/lib/POE/Wheel/Run.pm line 251
-  Compilation failed in require at t/30_loops/io_poll/wheel_run.t line 24.
-  # Looks like you planned 99 tests but ran 5.
-  # Looks like your test exited with 22 just after 5.
-  t/30_loops/io_poll/wheel_run.t ..................... Dubious, test returned 22 (wstat 5632, 0x1600)
-
 =head2 Other Compatibility Issues
 
 None currently known.  See GETTING HELP below if you've run into
@@ -483,6 +462,8 @@ an e-mail via his CPAN address.
 
 Broken down by abstraction layer.
 
+TODO - Verify this is complete.
+
 =head2 Layer 1
 
 L<POE::Kernel>, L<POE::Session>, L<POE::NFA>
@@ -514,27 +495,25 @@ L<POE::Queue>, L<POE::Queue::Array>
 
 L<POE::Resource>, L<POE::Resource::Aliases>, L<POE::Resource::Events>,
 L<POE::Resource::Extrefs>, L<POE::Resource::FileHandles>,
-L<POE::Resource::SIDs>, L<POE::Resource::Sessions>, L<POE::Resource::Signals>
+L<POE::Resource::Performance>, L<POE::Resource::SIDs>,
+L<POE::Resource::Sessions>, L<POE::Resource::Signals>
 
 =head2 Helpers
 
 L<POE::Pipe>, L<POE::Pipe::OneWay>, L<POE::Pipe::TwoWay>
 
-=head2 Home Page
+=head1 BUGS
 
-http://poe.perl.org/
+The tests only cover about 70% of POE.  Getting higher coverage is
+really hard.
 
-=head2 Bug Tracker
+Bug reports, suggestions, and feedback of all kinds should be e-mailed
+to <bug-POE@rt.cpan.org>.  They will be entered into our request queue
+and remain there until addressed.  If your return address is valid,
+you'll be notified when the status of your request changes.
 
-https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=POE
-
-=head2 Repository
-
-https://poe.svn.sourceforge.net/svnroot/poe/trunk/poe
-
-=head2 Other Resources
-
-http://search.cpan.org/dist/POE/
+Outstanding issues, including wish list items, are browsable in POE's
+RT queue at L<http://rt.cpan.org/>.
 
 =head1 AUTHORS & COPYRIGHT
 
@@ -567,9 +546,9 @@ future.
 
 =item Jos Boumans
 
-Jos Boumans is <kane@cpan.org>, aka C<kane>.  Jos is a major driving
-force behind the POE::Simple movement and has helped inspire the
-POE::Components for TCP clients and servers.
+Jos Boumans is <boumans@frg.eur.nl>, aka C<Co-Kane>.  Jos is a major
+driving force behind the POE::Simple movement and has helped inspire
+the POE::Components for TCP clients and servers.
 
 =item Matt Cashner
 
@@ -609,9 +588,6 @@ support different serialization methods.  He has also improved POE's quality
 by finding and fixing several bugs.  He provided POE a much needed code
 review around version 0.06.
 
-Lately, Philip tracked down the race condition in signal handling and
-fixed it with the signal pipe.
-
 =item Arnar M. Hrafnkelsson
 
 Arnar is <addi@umich.edu>.  Addi tested POE and L<POE::Component::IRC|POE::Component::IRC> on
@@ -649,7 +625,7 @@ problem relating to anonymous subs, scope and @{} processing.
 
 =item Matt Sergeant
 
-Matt contributed C<POE::Kernel::Poll>, a more efficient way to watch
+Matt contributed L<POE::Kernel::Poll|POE::Kernel::Poll>, a more efficient way to watch
 multiple files than select().  It's since been moved to
 L<POE::Loop::IO_Poll|POE::Loop::IO_Poll>.
 
@@ -694,7 +670,7 @@ to thank you for your interest, and he has more thanks than he can
 count for all the people who have contributed.  POE would not be
 nearly as cool without you.
 
-Except where otherwise noted, POE is Copyright 1998-2009 Rocco Caputo.
+Except where otherwise noted, POE is Copyright 1998-2007 Rocco Caputo.
 All rights reserved.  POE is free software; you may redistribute it
 and/or modify it under the same terms as Perl itself.
 
@@ -705,4 +681,3 @@ Thank you for reading!
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
