@@ -1,10 +1,12 @@
+# $Id: Block.pm 2221 2007-08-19 02:45:16Z rcaputo $
+
 package POE::Filter::Block;
 
 use strict;
 use POE::Filter;
 
 use vars qw($VERSION @ISA);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = do {my($r)=(q$Revision: 2221 $=~/(\d+)/);sprintf"1.%04d",$r};
 @ISA = qw(POE::Filter);
 
 use Carp qw(croak);
@@ -168,66 +170,50 @@ sub get_pending {
   [ $self->[FRAMING_BUFFER] ];
 }
 
+###############################################################################
 1;
 
 __END__
 
 =head1 NAME
 
-POE::Filter::Block - translate data between streams and blocks
+POE::Filter::Block - filter between streams and blocks
 
 =head1 SYNOPSIS
 
-  #!perl
-
-  use warnings;
-  use strict;
-  use POE::Filter::Block;
-
-  my $filter = POE::Filter::Block->new( BlockSize => 8 );
-
-  # Prints three lines: abcdefgh, ijklmnop, qrstuvwx.
-  # Bytes "y" and "z" remain in the buffer and await completion of the
-  # next 8-byte block.
-
-  $filter->get_one_start([ "abcdefghijklmnopqrstuvwxyz" ]);
-  while (1) {
-    my $block = $filter->get_one();
-    last unless @$block;
-    print $block->[0], "\n";
-  }
-
-  # Print one line: yz123456
-
-  $filter->get_one_start([ "123456" ]);
-  while (1) {
-    my $block = $filter->get_one();
-    last unless @$block;
-    print $block->[0], "\n";
-  }
+  $filter = POE::Filter::Block->new( BlockSize => 1024 );
+  $filter = POE::Filter::Block->new(
+    LengthCodec => [ \&encoder, \&decoder ]
+  );
+  $arrayref_of_blocks =
+    $filter->get($arrayref_of_raw_chunks_from_driver);
+  $arrayref_of_streamable_chunks_for_driver =
+    $filter->put($arrayref_of_blocks);
+  $arrayref_of_leftovers =
+    $filter->get_pending();
 
 =head1 DESCRIPTION
 
-POE::Filter::Block translates data between serial streams and blocks.
-It can handle fixed-length and length-prepended blocks, and it may be
-extended to handle other block types.
+The Block filter translates data between serial streams and blocks.
+It can handle two kinds of block: fixed-length and length-prepended.
 
 Fixed-length blocks are used when Block's constructor is called with a
 BlockSize value.  Otherwise the Block filter uses length-prepended
 blocks.
 
-Users who specify block sizes less than one deserve what they get.
+Users who specify block sizes less than one deserve to be soundly
+spanked.
 
-In variable-length mode, a LengthCodec parameter may be specified.
-The LengthCodec value should be a reference to a list of two
-functions: the length encoder, and the length decoder:
+In variable-length mode, a LengthCodec parameter is valid.  The
+LengthCodec should be a list reference of two functions: The length
+encoder, and the length decoder:
 
   LengthCodec => [ \&encoder, \&decoder ]
 
 The encoder takes a reference to a buffer and prepends the buffer's
 length to it.  The default encoder prepends the ASCII representation
-of the buffer's length and a chr(0) byte to separate the length from
-the actual data:
+of the buffer's length.  The length is separated from the buffer by an
+ASCII NUL ("\0") character.
 
   sub _default_encoder {
     my $stuff = shift;
@@ -235,9 +221,9 @@ the actual data:
     return;
   }
 
-The corresponding decoder returns the block length after removing it
-and the separator from the buffer.  It returns nothing if no length
-can be determined.
+Sensibly enough, the corresponding decoder removes the prepended
+length and separator, returning its numeric value.  It returns nothing
+if no length can be determined.
 
   sub _default_decoder {
     my $stuff = shift;
@@ -253,12 +239,11 @@ This filter holds onto incomplete blocks until they are completed.
 
 =head1 PUBLIC FILTER METHODS
 
-POE::Filter::Block has no additional public methods.
+Please see POE::Filter.
 
 =head1 SEE ALSO
 
-Please see L<POE::Filter> for documentation regarding the base
-interface.
+POE::Filter.
 
 The SEE ALSO section in L<POE> contains a table of contents covering
 the entire POE distribution.
@@ -277,4 +262,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
+# TODO - Redocument.
