@@ -1,3 +1,5 @@
+# $Id: PerlSignals.pm 2307 2008-03-31 11:46:07Z nothingmuch $
+
 # Plain Perl signal handling is something shared by several event
 # loops.  The invariant code has moved out here so that each loop may
 # use it without reinventing it.  This will save maintenance and
@@ -8,7 +10,7 @@ package POE::Loop::PerlSignals;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '1.294_512'; # NOTE - Should be #.### (three decimal places)
+$VERSION = do {my($r)=(q$Revision: 2307 $=~/(\d+)/);sprintf"1.%04d",$r};
 
 # Everything plugs into POE::Kernel.
 package POE::Kernel;
@@ -24,15 +26,6 @@ my %signal_watched;
 # Signal handlers/callbacks.
 
 sub _loop_signal_handler_generic {
-  if( USE_SIGNAL_PIPE ) {
-    POE::Kernel->_data_sig_pipe_send( $_[0] );
-  }
-  else {
-    _loop_signal_handler_generic_bottom( $_[0] );
-  }
-}
-
-sub _loop_signal_handler_generic_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing generic SIG$_[0] event";
   }
@@ -44,17 +37,7 @@ sub _loop_signal_handler_generic_bottom {
   $SIG{$_[0]} = \&_loop_signal_handler_generic;
 }
 
-##
 sub _loop_signal_handler_pipe {
-  if( USE_SIGNAL_PIPE ) {
-    POE::Kernel->_data_sig_pipe_send( $_[0] );
-  }
-  else {
-    _loop_signal_handler_pipe_bottom( $_[0] );
-  }
-}
-
-sub _loop_signal_handler_pipe_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing PIPE-like SIG$_[0] event";
   }
@@ -66,22 +49,14 @@ sub _loop_signal_handler_pipe_bottom {
   $SIG{$_[0]} = \&_loop_signal_handler_pipe;
 }
 
-## only used under USE_SIGCHLD
+# only used under USE_SIGCHLD
 sub _loop_signal_handler_chld {
-  if( USE_SIGNAL_PIPE ) {
-    POE::Kernel->_data_sig_pipe_send( 'CHLD' );
-  }
-  else {
-    _loop_signal_handler_chld_bottom( $_[0] );
-  }
-}
-
-sub _loop_signal_handler_chld_bottom {
   if (TRACE_SIGNALS) {
     POE::Kernel::_warn "<sg> Enqueuing CHLD-like SIG$_[0] event";
   }
 
-  $poe_kernel->_data_sig_enqueue_poll_event($_[0]);
+  $poe_kernel->_data_sig_enqueue_poll_event();
+  $SIG{$_[0]} = \&_loop_signal_handler_chld;
 }
 
 #------------------------------------------------------------------------------
@@ -95,13 +70,18 @@ sub loop_watch_signal {
   # Child process has stopped.
   if ($signal eq 'CHLD' or $signal eq 'CLD') {
     if ( USE_SIGCHLD ) {
-      # Poll once for signals.  Will set the signal handler when done.
-      $self->_data_sig_enqueue_poll_event($signal);
+      # install, but also trigger once
+      # there may be a race condition between forking, and $kernel->sig_chld in
+      # which the signal is already delivered
+      # and the interval polling mechanism will still generate a SIGCHLD
+      # signal, this preserves that behavior
+      $SIG{$signal} = \&_loop_signal_handler_chld;
+      $self->_data_sig_enqueue_poll_event();
     } else {
       # We should never twiddle $SIG{CH?LD} under POE, unless we want to
       # override system() and friends. --hachi
       # $SIG{$signal} = "DEFAULT";
-      $self->_data_sig_begin_polling($signal);
+      $self->_data_sig_begin_polling();
     }
     return;
   }
@@ -121,17 +101,8 @@ sub loop_ignore_signal {
 
   delete $signal_watched{$signal};
 
-  if ($signal eq 'CHLD' or $signal eq 'CLD') {
-    if ( USE_SIGCHLD ) {
-      if( $self->_data_sig_child_procs) {
-        # We need SIGCHLD to stay around after shutdown, so that
-        # child processes may be reaped and kr_child_procs=0
-        if (TRACE_SIGNALS) {
-          POE::Kernel::_warn "<sg> Keeping SIG$signal anyway!";
-        }
-        return;
-      }
-    } else {
+  unless ( USE_SIGCHLD ) {
+    if ($signal eq 'CHLD' or $signal eq 'CLD') {
       $self->_data_sig_cease_polling();
       # We should never twiddle $SIG{CH?LD} under poe, unless we want to
       # override system() and friends. --hachi
@@ -140,17 +111,12 @@ sub loop_ignore_signal {
     }
   }
 
-  delete $signal_watched{$signal};
-
-  my $state = 'DEFAULT';
   if ($signal eq 'PIPE') {
-    $state = "IGNORE";
+    $SIG{$signal} = "IGNORE";
+    return;
   }
 
-  if (TRACE_SIGNALS) {
-    POE::Kernel::_warn "<sg> $state SIG$signal";
-  }
-  $SIG{$signal} = $state;
+  $SIG{$signal} = "DEFAULT";
 }
 
 sub loop_ignore_all_signals {
@@ -166,7 +132,7 @@ __END__
 
 =head1 NAME
 
-POE::Loop::PerlSignals - common signal handling routines for POE::Loop bridges
+POE::Loop::PerlSignals - plain Perl signal handlers used by many loops
 
 =head1 SYNOPSIS
 
@@ -174,12 +140,10 @@ See L<POE::Loop>.
 
 =head1 DESCRIPTION
 
-POE::Loop::PerlSignals implements common code to handle signals for
-many different event loops.  Most loops don't handle signals natively,
-so this code has been abstracted into a reusable mix-in module.
-
-POE::Loop::PerlSignals follows POE::Loop's public interface for signal
-handling.  Therefore, please see L<POE::Loop> for more details.
+This class is an implementation of the signal handling functions
+defined by the abstract POE::Loop interface.  It follows POE::Loop's
+public interface for signal handling exactly.  Therefore, please see
+L<POE::Loop> for its documentation.
 
 =head1 SEE ALSO
 
@@ -193,4 +157,4 @@ and POE's licensing.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
-# TODO - Edit.
+# TODO - Redocument.
